/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2014 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without
any warranty. CUSTOMER retains the final decision relative to the total design
and functionality of the end product. FREESCALE neither guarantees nor will be
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING,
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED TO THE PROJECT
BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE SERVICES. IN NO EVENT
SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THIS AGREEMENT.
CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands or
actions by anyone on account of any damage, or injury, whether commercial,
contractual, or tortuous, rising directly or indirectly as a result of the advise
or assistance supplied CUSTOMER in connection with product, services or goods
supplied under this Agreement.
********************************************************************************
* File main.c
* Owner b35804
* Version 1.1
* Date May-16-2014
* Classification General Business Information
* FXAS21000 Gyro example code with use of FRDM-KL26Z and FRDM-FXS-MULTI boards
********************************************************************************
* Detailed Description: This code has been written for the Freescale FRDM-KL26Z 
* board and demonstrates how to read the angle acceleration data from the FXAS21000 
* 3-Axis Digital Rate Gyroscope using an interrupt technique.
* FXAS21000 is capable of measuring angular rates up to
* ±200°/s, with output data rates (ODR) from 1.5625 to 200 Hz.
* The device may be configured to generate an interrupt when a
* user-programmable angular rate threshold is crossed on any one
* of the enabled axes. 
********************************************************************************
Revision History:
Version		Date	 		Author		Description of Changes
1.0			May-02-2014		b35804		Gyro initialisation function zero condition
*******************************************************************************/

// Include peripheral declarations 
#include "derivative.h" 	// includes MKL26Z4.h			
#include "I2C.h"			
#include "FXAS21000.h"

/******************************************************************************
* Constants and macros
******************************************************************************/
//FXAS21000 7-bit I2C address
#define FXAS21_I2C_ADDRESS   0x20		// SA0 pin = 13 -> 7-bit slave 
										// I2C address is 0x20 = 0010 0000

//FXAS21000 Gyro Sensitivity at 0.2, 0.1, 0.05, 0.025 dps/LSB
#define SENGYR_2D		5		// LSB/dps = 0.2 dps/LSB
#define SENGYR_01D		10		// LSB/g = 0.1 dps/LSB
#define SENGYR_05D		20		// LSB/g = 0.05 dps/LSB
#define SENGYR_025D		40		// LSB/g = 0.025 dps/LSB

/******************************************************************************
* Global variables
******************************************************************************/
unsigned char GyrData[6];
short Xout_14_bit, Yout_14_bit, Zout_14_bit;
float Roll, Pitch, Yaw;
char DataReady;
signed char Temp;

/******************************************************************************
* Functions
******************************************************************************/
void MCU_Init(void);
void Gyro_Init (void);

/******************************************************************************
* Main
******************************************************************************/  
int main (void){
	DataReady = 0;
	MCU_Init();
  	Gyro_Init();   	
  	  	  					
  	while(1){
		if (DataReady){ 		// Is a new set of data ready?   		
			DataReady = 0;
																	
			I2C_ReadMultiRegisters(FXAS21_I2C_ADDRESS, OUT_X_MSB_REG, 6, GyrData);		// Read data output registers 0x01-0x06 
            
			Xout_14_bit = ((short) (GyrData[0]<<8 | GyrData[1])) >> 2;		// Compute 14-bit X-axis output value
			Yout_14_bit = ((short) (GyrData[2]<<8 | GyrData[3])) >> 2;		// Compute 14-bit Y-axis output value
			Zout_14_bit = ((short) (GyrData[4]<<8 | GyrData[5])) >> 2;		// Compute 14-bit Z-axis output value				
		
			Roll = ((float) (Xout_14_bit)) / SENGYR_025D;			// Compute X-axis output value in dps
			Pitch = ((float) (Yout_14_bit)) / SENGYR_025D;			// Compute Y-axis output value in dps
			Yaw = ((float) (Zout_14_bit)) / SENGYR_025D;			// Compute Z-axis output value in dps						
			
			Temp = (signed char) I2C_ReadRegister(FXAS21_I2C_ADDRESS, TEMP_REG);	// temperature on Gyro
		}                      
	}
}

/******************************************************************************
* MCU initialisation function
******************************************************************************/ 
void MCU_Init(void){
	//I2C1 module initialisation
	SIM_SCGC4 |= SIM_SCGC4_I2C1_MASK;		// Turn on clock to I2C1 module 
	SIM_SCGC5 |= SIM_SCGC5_PORTC_MASK;		// Turn on clock to Port C module 
	PORTC_PCR1 |= PORT_PCR_MUX(2);			// PTC1 pin is I2C1 SCL1 line pin alternative
	PORTC_PCR2 |= PORT_PCR_MUX(2);			// PTC2 pin is I2C1 SDA1 line pin alternative
	I2C1_F  = 0x14; 						// SDA hold time = 2.125us,  
											// SCL start hold time = 4.25us, 
											// SCL stop hold time = 5.125us
	I2C1_C1 = I2C_C1_IICEN_MASK;    		// Enable I2C1 module 
	
	//Configure the PTA5 pin (connected to the INT_GYRO of the FXAS21000) 
	//for falling edge interrupts
	SIM_SCGC5 |= SIM_SCGC5_PORTA_MASK;		// Turn on clock to Port A module 
	PORTA_PCR5 |= (0|PORT_PCR_ISF_MASK|		// Clear the interrupt flag 
					  PORT_PCR_MUX(0x1)|	// PTA5 is configured as GPIO 
					  PORT_PCR_IRQC(0xA));	// PTA5 is configured for falling edge interrupts 
	
	//Enable PORTA interrupt on NVIC
	NVIC_ICPR |= 1 << ((INT_PORTA - 16)%32); 
	NVIC_ISER |= 1 << ((INT_PORTA - 16)%32); 
}

/******************************************************************************
* Gyro initialisation function
******************************************************************************/ 
void Gyro_Init (void){
	unsigned char reg_val = 0;
	
	I2C_WriteRegister(FXAS21_I2C_ADDRESS, CTRL_REG1, 0x40);		// Reset all registers to POR values
	
	do		// Wait for the RST bit to clear 
	{
		reg_val = I2C_ReadRegister(FXAS21_I2C_ADDRESS, CTRL_REG1) & 0x40;
	} 	while (reg_val);
	
	// Zero values initialisation ------------------------------------------------------------
	//	I2C_WriteRegister(FXAS21_I2C_ADDRESS, CTRL_REG1, 0x80);	// ZR_cond to trigger offset compensation
	do		// wait till ZR_cond to trigger offset compensation accomplished
	{
		reg_val = I2C_ReadRegister(FXAS21_I2C_ADDRESS, CTRL_REG1) & 0x80;	
	} 	while (reg_val);
	//----------------------------------------------------------------------------------------
	
	I2C_WriteRegister(FXAS21_I2C_ADDRESS, CTRL_REG2, 0x0C);		// Enable DRDY interrupt, DRDY interrupt routed to INT1 - PTA5, Push-pull, active low interrupt 
	I2C_WriteRegister(FXAS21_I2C_ADDRESS, CTRL_REG0, 0x17);		// High-pass filter enabled, H-P filter cutoff frq.:0.047 Hz, +/-200 dps range -> 0.025 dsp/LSB = 40 LSB/dps 
	I2C_WriteRegister(FXAS21_I2C_ADDRESS, CTRL_REG1, 0x1E);		// ODR = 1.5625Hz(640ms), Active mode
}

/******************************************************************************
* PORT A Interrupt handler
******************************************************************************/ 
void PORTA_IRQHandler(){
	PORTA_PCR5 |= PORT_PCR_ISF_MASK;			// Clear the interrupt flag 
	DataReady = 1;	
}
